"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _runtimeDom = require("@vue/runtime-dom");

var _cloneDeep = _interopRequireDefault(require("lodash-es/cloneDeep"));

var _intersection = _interopRequireDefault(require("lodash-es/intersection"));

var _isEqual = _interopRequireDefault(require("lodash-es/isEqual"));

var _debounce = _interopRequireDefault(require("lodash-es/debounce"));

var _omit = _interopRequireDefault(require("lodash-es/omit"));

var _validateUtil = require("ant-design-vue/es/form/utils/validateUtil");

var _messages = require("ant-design-vue/es/form/utils/messages");

var _asyncUtil = require("ant-design-vue/es/form/utils/asyncUtil");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? Object(arguments[i]) : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function isRequired(rules) {
  var isRequired = false;

  if (rules && rules.length) {
    rules.every(function (rule) {
      if (rule.required) {
        isRequired = true;
        return false;
      }

      return true;
    });
  }

  return isRequired;
}

function toArray(value) {
  if (value === undefined || value === null) {
    return [];
  }

  return Array.isArray(value) ? value : [value];
} // const isArray = Array.isArray;
// const isObject = (val: any) => val !== null && typeof val === 'object';
// 重置到初始数据，并尽可能的保留响应式
// function resetReactiveValue(originValue: Props, refValues: Props) {
//   for (const key of Object.keys(refValues)) {
//     if (!(key in originValue)) {
//       delete refValues[key];
//     }
//   }
//   for (const [key, value] of Object.entries(originValue)) {
//     const refValue = refValues[key];
//     if (isArray(value) && isArray(refValue)) {
//       if (value.length <= refValue.length) {
//         refValue.splice(value.length, refValue.length - value.length);
//       } else {
//         refValue.push(...value.slice(refValue.length));
//       }
//       value.forEach((val, index) => {
//         refValues[key][index] = resetReactiveValue(val, refValue[index]);
//       });
//     } else if (isObject(value) && isObject(refValue)) {
//       refValues[key] = resetReactiveValue(value, refValue);
//     } else {
//       refValues[key] = value;
//     }
//   }
//   return refValues;
// }


function getPropByPath(obj, path, strict) {
  var tempObj = obj;
  path = path.replace(/\[(\w+)\]/g, '.$1');
  path = path.replace(/^\./, '');
  var keyArr = path.split('.');
  var i = 0;

  for (var len = keyArr.length; i < len - 1; ++i) {
    if (!tempObj && !strict) break;
    var key = keyArr[i];

    if (key in tempObj) {
      tempObj = tempObj[key];
    } else {
      if (strict) {
        throw new Error('please transfer a valid name path to validate!');
      }

      break;
    }
  }

  return {
    o: tempObj,
    k: keyArr[i],
    v: tempObj ? tempObj[keyArr[i]] : null,
    isValid: tempObj && keyArr[i] in tempObj
  };
}

function useForm(modelRef, rulesRef, options) {
  var initialModel = (0, _cloneDeep["default"])(modelRef);
  var validateInfos = {};
  Object.keys(rulesRef).forEach(function (key) {
    validateInfos[key] = {
      autoLink: false,
      required: isRequired(rulesRef[key])
    };
  });
  validateInfos = (0, _runtimeDom.reactive)(validateInfos);

  var resetFields = function resetFields() {
    _extends(modelRef, (0, _cloneDeep["default"])(initialModel)); //modelRef = resetReactiveValue(initialModel, modelRef);


    (0, _runtimeDom.nextTick)(function () {
      Object.keys(validateInfos).forEach(function (key) {
        validateInfos[key] = {
          autoLink: false,
          required: isRequired(rulesRef[key])
        };
      });
    });
  };

  var filterRules = function filterRules() {
    var rules = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    var trigger = arguments.length > 1 ? arguments[1] : undefined;

    if (!trigger.length) {
      return rules;
    } else {
      return rules.filter(function (rule) {
        var triggerList = toArray(rule.trigger || 'change');
        return (0, _intersection["default"])(triggerList, trigger).length;
      });
    }
  };

  var lastValidatePromise = null;

  var validateFields = function validateFields(names) {
    var option = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var strict = arguments.length > 2 ? arguments[2] : undefined;
    var promiseList = [];
    var values = {};

    var _loop = function _loop(i) {
      var name = names[i];
      var prop = getPropByPath(modelRef, name, strict);
      if (!prop.isValid) return "continue";
      values[name] = prop.v;
      var rules = filterRules(rulesRef[name], toArray(option && option.trigger));

      if (rules.length) {
        promiseList.push(validateField(name, prop.v, rules, option || {}).then(function () {
          return {
            name: name,
            errors: []
          };
        })["catch"](function (errors) {
          return Promise.reject({
            name: name,
            errors: errors
          });
        }));
      }
    };

    for (var i = 0; i < names.length; i++) {
      var _ret = _loop(i);

      if (_ret === "continue") continue;
    }

    var summaryPromise = (0, _asyncUtil.allPromiseFinish)(promiseList);
    lastValidatePromise = summaryPromise;
    var returnPromise = summaryPromise.then(function () {
      if (lastValidatePromise === summaryPromise) {
        return Promise.resolve(values);
      }

      return Promise.reject([]);
    })["catch"](function (results) {
      var errorList = results.filter(function (result) {
        return result && result.errors.length;
      });
      return Promise.reject({
        values: values,
        errorFields: errorList,
        outOfDate: lastValidatePromise !== summaryPromise
      });
    }); // Do not throw in console

    returnPromise["catch"](function (e) {
      return e;
    });
    return returnPromise;
  };

  var validateField = function validateField(name, value, rules, option) {
    var promise = (0, _validateUtil.validateRules)([name], value, rules, _objectSpread({
      validateMessages: _messages.defaultValidateMessages
    }, option), !!option.validateFirst);
    validateInfos[name].validateStatus = 'validating';
    promise["catch"](function (e) {
      return e;
    }).then(function () {
      var errors = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

      if (validateInfos[name].validateStatus === 'validating') {
        validateInfos[name].validateStatus = errors.length ? 'error' : 'success';
        validateInfos[name].help = errors[0];
      }
    });
    return promise;
  };

  var validate = function validate(names, option) {
    var keys = [];
    var strict = true;

    if (!names) {
      strict = false;
      keys = Object.keys(rulesRef);
    } else if (Array.isArray(names)) {
      keys = names;
    } else {
      keys = [names];
    }

    var promises = validateFields(keys, option || {}, strict); // Do not throw in console

    promises["catch"](function (e) {
      return e;
    });
    return promises;
  };

  var mergeValidateInfo = function mergeValidateInfo() {
    var items = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    var info = {
      autoLink: false
    };
    var help = [];
    var infos = Array.isArray(items) ? items : [items];

    for (var i = 0; i < infos.length; i++) {
      var arg = infos[i];

      if (arg?.validateStatus === 'error') {
        info.validateStatus = 'error';
        arg.help && help.push(arg.help);
      }

      info.required = info.required || arg?.required;
    }

    info.help = help.join('\n');
    return info;
  };

  var oldModel = initialModel;

  var modelFn = function modelFn(model) {
    var names = [];
    Object.keys(rulesRef).forEach(function (key) {
      var prop = getPropByPath(model, key, false);
      var oldProp = getPropByPath(oldModel, key, false);

      if (!(0, _isEqual["default"])(prop.v, oldProp.v)) {
        names.push(key);
      }
    });
    validate(names, {
      trigger: 'change'
    });
    oldModel = (0, _cloneDeep["default"])(model);
  };

  var debounceOptions = options?.debounce;
  (0, _runtimeDom.watch)(modelRef, debounceOptions && debounceOptions.wait ? (0, _debounce["default"])(modelFn, debounceOptions.wait, (0, _omit["default"])(debounceOptions, ['wait'])) : modelFn, {
    immediate: options && !!options.immediate,
    deep: true
  });
  (0, _runtimeDom.watch)(rulesRef, function () {
    if (options && options.validateOnRuleChange) {
      validate();
    }
  }, {
    deep: true
  });
  return {
    modelRef: modelRef,
    rulesRef: rulesRef,
    initialModel: initialModel,
    validateInfos: validateInfos,
    resetFields: resetFields,
    validate: validate,
    validateField: validateField,
    mergeValidateInfo: mergeValidateInfo
  };
}

var _default = useForm;
exports["default"] = _default;